/*
 * Tencent is pleased to support the open source community by making Blueking Container Service available.
 * Copyright (C) 2019 THL A29 Limited, a Tencent company. All rights reserved.
 * Licensed under the MIT License (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * http://opensource.org/licenses/MIT
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

package versions

//all apiVersion supported list
var apiVersionMap = map[string][]string{
	"1.5":   apiSetV15,
	"1.6":   apiSetV16,
	"1.7":   apiSetV17,
	"1.8":   apiSetV18,
	"1.11":  apiSetV111,
	"1.12":  apiSetV112,
	"1.12+": apiSetV112,
	"1.13":  apiSetV112,
	"1.13+": apiSetV112,
	"1.14":  apiSetV112,
	"1.14+": apiSetV112,
	"1.15":  apiSetV112,
	"1.16":  apiSetV112,
	"1.17":  apiSetV112,
	"1.18":  apiSetV112,
	"1.19":  apiSetV112,
	"1.20":  apiSetV112,
}

var apiSetV15 = []string{
	"/api/v1/componentstatuses",
	"/api/v1/componentstatuses/{name}",
	"/api/v1/configmaps",
	"/api/v1/endpoints",
	"/api/v1/events",
	"/api/v1/limitranges",
	"/api/v1/namespaces",
	"/api/v1/namespaces/{namespace}/bindings",
	"/api/v1/namespaces/{namespace}/configmaps",
	"/api/v1/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/namespaces/{namespace}/endpoints",
	"/api/v1/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/namespaces/{namespace}/events",
	"/api/v1/namespaces/{namespace}/events/{name}",
	"/api/v1/namespaces/{namespace}/limitranges",
	"/api/v1/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}/status",
	"/api/v1/namespaces/{namespace}/pods",
	"/api/v1/namespaces/{namespace}/pods/{name}",
	"/api/v1/namespaces/{namespace}/pods/{name}/attach",
	"/api/v1/namespaces/{namespace}/pods/{name}/binding",
	"/api/v1/namespaces/{namespace}/pods/{name}/eviction",
	"/api/v1/namespaces/{namespace}/pods/{name}/exec",
	"/api/v1/namespaces/{namespace}/pods/{name}/log",
	"/api/v1/namespaces/{namespace}/pods/{name}/portforward",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/pods/{name}/status",
	"/api/v1/namespaces/{namespace}/podtemplates",
	"/api/v1/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/status",
	"/api/v1/namespaces/{namespace}/resourcequotas",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}/status",
	"/api/v1/namespaces/{namespace}/secrets",
	"/api/v1/namespaces/{namespace}/secrets/{name}",
	"/api/v1/namespaces/{namespace}/serviceaccounts",
	"/api/v1/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/namespaces/{namespace}/services",
	"/api/v1/namespaces/{namespace}/services/{name}",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/services/{name}/status",
	"/api/v1/namespaces/{namespace}",
	"/api/v1/namespaces/{namespace}/finalize",
	"/api/v1/namespaces/{namespace}/status",
	"/api/v1/nodes",
	"/api/v1/nodes/{name}",
	"/api/v1/nodes/{name}/proxy",
	"/api/v1/nodes/{name}/proxy/{path}",
	"/api/v1/nodes/{name}/status",
	"/api/v1/persistentvolumeclaims",
	"/api/v1/persistentvolumes",
	"/api/v1/persistentvolumes/{name}",
	"/api/v1/persistentvolumes/{name}/status",
	"/api/v1/pods",
	"/api/v1/podtemplates",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}/{path}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}/{path}",
	"/api/v1/proxy/nodes/{name}",
	"/api/v1/proxy/nodes/{name}/{path}",
	"/api/v1/replicationcontrollers",
	"/api/v1/resourcequotas",
	"/api/v1/secrets",
	"/api/v1/serviceaccounts",
	"/api/v1/services",
	"/api/v1/watch/configmaps",
	"/api/v1/watch/endpoints",
	"/api/v1/watch/events",
	"/api/v1/watch/limitranges",
	"/api/v1/watch/namespaces",
	"/api/v1/watch/namespaces/{namespace}/configmaps",
	"/api/v1/watch/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/watch/namespaces/{namespace}/endpoints",
	"/api/v1/watch/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/watch/namespaces/{namespace}/events",
	"/api/v1/watch/namespaces/{namespace}/events/{name}",
	"/api/v1/watch/namespaces/{namespace}/limitranges",
	"/api/v1/watch/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/watch/namespaces/{namespace}/pods",
	"/api/v1/watch/namespaces/{namespace}/pods/{name}",
	"/api/v1/watch/namespaces/{namespace}/podtemplates",
	"/api/v1/watch/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/watch/namespaces/{namespace}/secrets",
	"/api/v1/watch/namespaces/{namespace}/secrets/{name}",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/watch/namespaces/{namespace}/services",
	"/api/v1/watch/namespaces/{namespace}/services/{name}",
	"/api/v1/watch/namespaces/{namespace}",
	"/api/v1/watch/nodes",
	"/api/v1/watch/nodes/{name}",
	"/api/v1/watch/persistentvolumeclaims",
	"/api/v1/watch/persistentvolumes",
	"/api/v1/watch/persistentvolumes/{name}",
	"/api/v1/watch/pods",
	"/api/v1/watch/podtemplates",
	"/api/v1/watch/replicationcontrollers",
	"/api/v1/watch/resourcequotas",
	"/api/v1/watch/secrets",
	"/api/v1/watch/serviceaccounts",
	"/api/v1/watch/services",
	"/apis/",
	"/apis/apps/",
	"/apis/apps/v1beta1/",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}/status",
	"/apis/apps/v1beta1/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/watch/statefulsets",
	"/apis/authentication.k8s.io/",
	"/apis/authentication.k8s.io/v1beta1/",
	"/apis/authentication.k8s.io/v1beta1/tokenreviews",
	"/apis/authorization.k8s.io/",
	"/apis/authorization.k8s.io/v1beta1/",
	"/apis/authorization.k8s.io/v1beta1/namespaces/{namespace}/localsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/selfsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/subjectaccessreviews",
	"/apis/autoscaling/",
	"/apis/autoscaling/v1/",
	"/apis/autoscaling/v1/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/autoscaling/v1/watch/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/batch/",
	"/apis/batch/v1/",
	"/apis/batch/v1/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v1/watch/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/",
	"/apis/batch/v2alpha1/cronjobs",
	"/apis/batch/v2alpha1/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}/status",
	"/apis/batch/v2alpha1/scheduledjobs",
	"/apis/batch/v2alpha1/watch/cronjobs",
	"/apis/batch/v2alpha1/watch/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/watch/scheduledjobs",
	"/apis/certificates.k8s.io/",
	"/apis/certificates.k8s.io/v1alpha1/",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/approval",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/status",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests/{name}",
	"/apis/extensions/",
	"/apis/extensions/v1beta1/",
	"/apis/extensions/v1beta1/daemonsets",
	"/apis/extensions/v1beta1/deployments",
	"/apis/extensions/v1beta1/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/ingresses",
	"/apis/extensions/v1beta1/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/rollback",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/apis/extensions/v1beta1/networkpolicies",
	"/apis/extensions/v1beta1/replicasets",
	"/apis/extensions/v1beta1/thirdpartyresources",
	"/apis/extensions/v1beta1/thirdpartyresources/{name}",
	"/apis/extensions/v1beta1/watch/daemonsets",
	"/apis/extensions/v1beta1/watch/deployments",
	"/apis/extensions/v1beta1/watch/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/ingresses",
	"/apis/extensions/v1beta1/watch/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/watch/networkpolicies",
	"/apis/extensions/v1beta1/watch/replicasets",
	"/apis/extensions/v1beta1/watch/thirdpartyresources",
	"/apis/extensions/v1beta1/watch/thirdpartyresources/{name}",
	"/apis/policy/",
	"/apis/policy/v1beta1/",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}/status",
	"/apis/policy/v1beta1/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/watch/poddisruptionbudgets",
	"/apis/rbac.authorization.k8s.io/",
	"/apis/rbac.authorization.k8s.io/v1alpha1/",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/roles",
	"/apis/storage.k8s.io/",
	"/apis/storage.k8s.io/v1beta1/",
	"/apis/storage.k8s.io/v1beta1/storageclasses",
	"/apis/storage.k8s.io/v1beta1/storageclasses/{name}",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses/{name}",
	"/logs/",
	"/logs/{logpath}",
	"/version/",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/{name}",
	"/apis/metrics.k8s.io/v1beta1/pods",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/",
	"/apis/metrics.k8s.io/v1beta1/nodes",
	"/apis/metrics.k8s.io/v1beta1/nodes/{name}",
}
var apiSetV16 = []string{
	"/api/v1/componentstatuses",
	"/api/v1/componentstatuses/{name}",
	"/api/v1/configmaps",
	"/api/v1/endpoints",
	"/api/v1/events",
	"/api/v1/limitranges",
	"/api/v1/namespaces",
	"/api/v1/namespaces/{namespace}/bindings",
	"/api/v1/namespaces/{namespace}/configmaps",
	"/api/v1/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/namespaces/{namespace}/endpoints",
	"/api/v1/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/namespaces/{namespace}/events",
	"/api/v1/namespaces/{namespace}/events/{name}",
	"/api/v1/namespaces/{namespace}/limitranges",
	"/api/v1/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}/status",
	"/api/v1/namespaces/{namespace}/pods",
	"/api/v1/namespaces/{namespace}/pods/{name}",
	"/api/v1/namespaces/{namespace}/pods/{name}/attach",
	"/api/v1/namespaces/{namespace}/pods/{name}/binding",
	"/api/v1/namespaces/{namespace}/pods/{name}/eviction",
	"/api/v1/namespaces/{namespace}/pods/{name}/exec",
	"/api/v1/namespaces/{namespace}/pods/{name}/log",
	"/api/v1/namespaces/{namespace}/pods/{name}/portforward",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/pods/{name}/status",
	"/api/v1/namespaces/{namespace}/podtemplates",
	"/api/v1/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/status",
	"/api/v1/namespaces/{namespace}/resourcequotas",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}/status",
	"/api/v1/namespaces/{namespace}/secrets",
	"/api/v1/namespaces/{namespace}/secrets/{name}",
	"/api/v1/namespaces/{namespace}/serviceaccounts",
	"/api/v1/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/namespaces/{namespace}/services",
	"/api/v1/namespaces/{namespace}/services/{name}",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/services/{name}/status",
	"/api/v1/namespaces/{namespace}",
	"/api/v1/namespaces/{namespace}/finalize",
	"/api/v1/namespaces/{namespace}/status",
	"/api/v1/nodes",
	"/api/v1/nodes/{name}",
	"/api/v1/nodes/{name}/proxy",
	"/api/v1/nodes/{name}/proxy/{path}",
	"/api/v1/nodes/{name}/status",
	"/api/v1/persistentvolumeclaims",
	"/api/v1/persistentvolumes",
	"/api/v1/persistentvolumes/{name}",
	"/api/v1/persistentvolumes/{name}/status",
	"/api/v1/pods",
	"/api/v1/podtemplates",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}/{path}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}/{path}",
	"/api/v1/proxy/nodes/{name}",
	"/api/v1/proxy/nodes/{name}/{path}",
	"/api/v1/replicationcontrollers",
	"/api/v1/resourcequotas",
	"/api/v1/secrets",
	"/api/v1/serviceaccounts",
	"/api/v1/services",
	"/api/v1/watch/configmaps",
	"/api/v1/watch/endpoints",
	"/api/v1/watch/events",
	"/api/v1/watch/limitranges",
	"/api/v1/watch/namespaces",
	"/api/v1/watch/namespaces/{namespace}/configmaps",
	"/api/v1/watch/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/watch/namespaces/{namespace}/endpoints",
	"/api/v1/watch/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/watch/namespaces/{namespace}/events",
	"/api/v1/watch/namespaces/{namespace}/events/{name}",
	"/api/v1/watch/namespaces/{namespace}/limitranges",
	"/api/v1/watch/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/watch/namespaces/{namespace}/pods",
	"/api/v1/watch/namespaces/{namespace}/pods/{name}",
	"/api/v1/watch/namespaces/{namespace}/podtemplates",
	"/api/v1/watch/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/watch/namespaces/{namespace}/secrets",
	"/api/v1/watch/namespaces/{namespace}/secrets/{name}",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/watch/namespaces/{namespace}/services",
	"/api/v1/watch/namespaces/{namespace}/services/{name}",
	"/api/v1/watch/namespaces/{namespace}",
	"/api/v1/watch/nodes",
	"/api/v1/watch/nodes/{name}",
	"/api/v1/watch/persistentvolumeclaims",
	"/api/v1/watch/persistentvolumes",
	"/api/v1/watch/persistentvolumes/{name}",
	"/api/v1/watch/pods",
	"/api/v1/watch/podtemplates",
	"/api/v1/watch/replicationcontrollers",
	"/api/v1/watch/resourcequotas",
	"/api/v1/watch/secrets",
	"/api/v1/watch/serviceaccounts",
	"/api/v1/watch/services",
	"/apis/apps/v1beta1/",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}/status",
	"/apis/apps/v1beta1/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/watch/statefulsets",
	"/apis/authentication.k8s.io/v1beta1/",
	"/apis/authentication.k8s.io/v1beta1/tokenreviews",
	"/apis/authorization.k8s.io/v1beta1/",
	"/apis/authorization.k8s.io/v1beta1/namespaces/{namespace}/localsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/selfsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/subjectaccessreviews",
	"/apis/autoscaling/v1/",
	"/apis/autoscaling/v1/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/autoscaling/v1/watch/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/batch/v1/",
	"/apis/batch/v1/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v1/watch/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/",
	"/apis/batch/v2alpha1/cronjobs",
	"/apis/batch/v2alpha1/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}/status",
	"/apis/batch/v2alpha1/scheduledjobs",
	"/apis/batch/v2alpha1/watch/cronjobs",
	"/apis/batch/v2alpha1/watch/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/watch/scheduledjobs",
	"/apis/certificates.k8s.io/v1alpha1/",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/approval",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/status",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests/{name}",
	"/apis/extensions/v1beta1/",
	"/apis/extensions/v1beta1/daemonsets",
	"/apis/extensions/v1beta1/deployments",
	"/apis/extensions/v1beta1/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/ingresses",
	"/apis/extensions/v1beta1/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/rollback",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/apis/extensions/v1beta1/networkpolicies",
	"/apis/extensions/v1beta1/replicasets",
	"/apis/extensions/v1beta1/thirdpartyresources",
	"/apis/extensions/v1beta1/thirdpartyresources/{name}",
	"/apis/extensions/v1beta1/watch/daemonsets",
	"/apis/extensions/v1beta1/watch/deployments",
	"/apis/extensions/v1beta1/watch/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/ingresses",
	"/apis/extensions/v1beta1/watch/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/watch/networkpolicies",
	"/apis/extensions/v1beta1/watch/replicasets",
	"/apis/extensions/v1beta1/watch/thirdpartyresources",
	"/apis/extensions/v1beta1/watch/thirdpartyresources/{name}",
	"/apis/policy/v1beta1/",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}/status",
	"/apis/policy/v1beta1/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/watch/poddisruptionbudgets",
	"/apis/rbac.authorization.k8s.io/v1alpha1/",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/roles",
	"/apis/storage.k8s.io/v1beta1/",
	"/apis/storage.k8s.io/v1beta1/storageclasses",
	"/apis/storage.k8s.io/v1beta1/storageclasses/{name}",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses/{name}",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/{name}",
	"/apis/metrics.k8s.io/v1beta1/pods",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/",
	"/apis/metrics.k8s.io/v1beta1/nodes",
	"/apis/metrics.k8s.io/v1beta1/nodes/{name}",
}
var apiSetV17 = []string{
	"/api/v1/",
	"/api/v1/componentstatuses",
	"/api/v1/componentstatuses/{name}",
	"/api/v1/configmaps",
	"/api/v1/endpoints",
	"/api/v1/events",
	"/api/v1/limitranges",
	"/api/v1/namespaces",
	"/api/v1/namespaces/{namespace}/bindings",
	"/api/v1/namespaces/{namespace}/configmaps",
	"/api/v1/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/namespaces/{namespace}/endpoints",
	"/api/v1/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/namespaces/{namespace}/events",
	"/api/v1/namespaces/{namespace}/events/{name}",
	"/api/v1/namespaces/{namespace}/limitranges",
	"/api/v1/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}/status",
	"/api/v1/namespaces/{namespace}/pods",
	"/api/v1/namespaces/{namespace}/pods/{name}",
	"/api/v1/namespaces/{namespace}/pods/{name}/attach",
	"/api/v1/namespaces/{namespace}/pods/{name}/binding",
	"/api/v1/namespaces/{namespace}/pods/{name}/eviction",
	"/api/v1/namespaces/{namespace}/pods/{name}/exec",
	"/api/v1/namespaces/{namespace}/pods/{name}/log",
	"/api/v1/namespaces/{namespace}/pods/{name}/portforward",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/pods/{name}/status",
	"/api/v1/namespaces/{namespace}/podtemplates",
	"/api/v1/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/status",
	"/api/v1/namespaces/{namespace}/resourcequotas",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}/status",
	"/api/v1/namespaces/{namespace}/secrets",
	"/api/v1/namespaces/{namespace}/secrets/{name}",
	"/api/v1/namespaces/{namespace}/serviceaccounts",
	"/api/v1/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/namespaces/{namespace}/services",
	"/api/v1/namespaces/{namespace}/services/{name}",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/services/{name}/status",
	"/api/v1/namespaces/{namespace}",
	"/api/v1/namespaces/{namespace}/finalize",
	"/api/v1/namespaces/{namespace}/status",
	"/api/v1/nodes",
	"/api/v1/nodes/{name}",
	"/api/v1/nodes/{name}/proxy",
	"/api/v1/nodes/{name}/proxy/{path}",
	"/api/v1/nodes/{name}/status",
	"/api/v1/persistentvolumeclaims",
	"/api/v1/persistentvolumes",
	"/api/v1/persistentvolumes/{name}",
	"/api/v1/persistentvolumes/{name}/status",
	"/api/v1/pods",
	"/api/v1/podtemplates",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}/{path}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}/{path}",
	"/api/v1/proxy/nodes/{name}",
	"/api/v1/proxy/nodes/{name}/{path}",
	"/api/v1/replicationcontrollers",
	"/api/v1/resourcequotas",
	"/api/v1/secrets",
	"/api/v1/serviceaccounts",
	"/api/v1/services",
	"/api/v1/watch/configmaps",
	"/api/v1/watch/endpoints",
	"/api/v1/watch/events",
	"/api/v1/watch/limitranges",
	"/api/v1/watch/namespaces",
	"/api/v1/watch/namespaces/{namespace}/configmaps",
	"/api/v1/watch/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/watch/namespaces/{namespace}/endpoints",
	"/api/v1/watch/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/watch/namespaces/{namespace}/events",
	"/api/v1/watch/namespaces/{namespace}/events/{name}",
	"/api/v1/watch/namespaces/{namespace}/limitranges",
	"/api/v1/watch/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/watch/namespaces/{namespace}/pods",
	"/api/v1/watch/namespaces/{namespace}/pods/{name}",
	"/api/v1/watch/namespaces/{namespace}/podtemplates",
	"/api/v1/watch/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/watch/namespaces/{namespace}/secrets",
	"/api/v1/watch/namespaces/{namespace}/secrets/{name}",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/watch/namespaces/{namespace}/services",
	"/api/v1/watch/namespaces/{namespace}/services/{name}",
	"/api/v1/watch/namespaces/{namespace}",
	"/api/v1/watch/nodes",
	"/api/v1/watch/nodes/{name}",
	"/api/v1/watch/persistentvolumeclaims",
	"/api/v1/watch/persistentvolumes",
	"/api/v1/watch/persistentvolumes/{name}",
	"/api/v1/watch/pods",
	"/api/v1/watch/podtemplates",
	"/api/v1/watch/replicationcontrollers",
	"/api/v1/watch/resourcequotas",
	"/api/v1/watch/secrets",
	"/api/v1/watch/serviceaccounts",
	"/api/v1/watch/services",
	"/apis/",
	"/apis/apps/",
	"/apis/apps/v1beta1/",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}/status",
	"/apis/apps/v1beta1/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/watch/statefulsets",
	"/apis/authentication.k8s.io/",
	"/apis/authentication.k8s.io/v1beta1/",
	"/apis/authentication.k8s.io/v1beta1/tokenreviews",
	"/apis/authorization.k8s.io/",
	"/apis/authorization.k8s.io/v1beta1/",
	"/apis/authorization.k8s.io/v1beta1/namespaces/{namespace}/localsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/selfsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/subjectaccessreviews",
	"/apis/autoscaling/",
	"/apis/autoscaling/v1/",
	"/apis/autoscaling/v1/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/autoscaling/v1/watch/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/batch/",
	"/apis/batch/v1/",
	"/apis/batch/v1/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v1/watch/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/",
	"/apis/batch/v2alpha1/cronjobs",
	"/apis/batch/v2alpha1/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}/status",
	"/apis/batch/v2alpha1/scheduledjobs",
	"/apis/batch/v2alpha1/watch/cronjobs",
	"/apis/batch/v2alpha1/watch/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/watch/scheduledjobs",
	"/apis/certificates.k8s.io/",
	"/apis/certificates.k8s.io/v1alpha1/",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/approval",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/status",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests/{name}",
	"/apis/extensions/",
	"/apis/extensions/v1beta1/",
	"/apis/extensions/v1beta1/daemonsets",
	"/apis/extensions/v1beta1/deployments",
	"/apis/extensions/v1beta1/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/ingresses",
	"/apis/extensions/v1beta1/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/rollback",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/apis/extensions/v1beta1/networkpolicies",
	"/apis/extensions/v1beta1/replicasets",
	"/apis/extensions/v1beta1/thirdpartyresources",
	"/apis/extensions/v1beta1/thirdpartyresources/{name}",
	"/apis/extensions/v1beta1/watch/daemonsets",
	"/apis/extensions/v1beta1/watch/deployments",
	"/apis/extensions/v1beta1/watch/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/ingresses",
	"/apis/extensions/v1beta1/watch/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/watch/networkpolicies",
	"/apis/extensions/v1beta1/watch/replicasets",
	"/apis/extensions/v1beta1/watch/thirdpartyresources",
	"/apis/extensions/v1beta1/watch/thirdpartyresources/{name}",
	"/apis/policy/",
	"/apis/policy/v1beta1/",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}/status",
	"/apis/policy/v1beta1/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/watch/poddisruptionbudgets",
	"/apis/rbac.authorization.k8s.io/",
	"/apis/rbac.authorization.k8s.io/v1alpha1/",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/roles",
	"/apis/storage.k8s.io/",
	"/apis/storage.k8s.io/v1beta1/",
	"/apis/storage.k8s.io/v1beta1/storageclasses",
	"/apis/storage.k8s.io/v1beta1/storageclasses/{name}",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses/{name}",
	"/logs/",
	"/logs/{logpath}",
	"/version/",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/{name}",
	"/apis/metrics.k8s.io/v1beta1/pods",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/",
	"/apis/metrics.k8s.io/v1beta1/nodes",
	"/apis/metrics.k8s.io/v1beta1/nodes/{name}",
}
var apiSetV18 = []string{
	"/api/v1/",
	"/api/v1/componentstatuses",
	"/api/v1/componentstatuses/{name}",
	"/api/v1/configmaps",
	"/api/v1/endpoints",
	"/api/v1/events",
	"/api/v1/limitranges",
	"/api/v1/namespaces",
	"/api/v1/namespaces/{namespace}/bindings",
	"/api/v1/namespaces/{namespace}/configmaps",
	"/api/v1/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/namespaces/{namespace}/endpoints",
	"/api/v1/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/namespaces/{namespace}/events",
	"/api/v1/namespaces/{namespace}/events/{name}",
	"/api/v1/namespaces/{namespace}/limitranges",
	"/api/v1/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}/status",
	"/api/v1/namespaces/{namespace}/pods",
	"/api/v1/namespaces/{namespace}/pods/{name}",
	"/api/v1/namespaces/{namespace}/pods/{name}/attach",
	"/api/v1/namespaces/{namespace}/pods/{name}/binding",
	"/api/v1/namespaces/{namespace}/pods/{name}/eviction",
	"/api/v1/namespaces/{namespace}/pods/{name}/exec",
	"/api/v1/namespaces/{namespace}/pods/{name}/log",
	"/api/v1/namespaces/{namespace}/pods/{name}/portforward",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/pods/{name}/status",
	"/api/v1/namespaces/{namespace}/podtemplates",
	"/api/v1/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/status",
	"/api/v1/namespaces/{namespace}/resourcequotas",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}/status",
	"/api/v1/namespaces/{namespace}/secrets",
	"/api/v1/namespaces/{namespace}/secrets/{name}",
	"/api/v1/namespaces/{namespace}/serviceaccounts",
	"/api/v1/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/namespaces/{namespace}/services",
	"/api/v1/namespaces/{namespace}/services/{name}",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/services/{name}/status",
	"/api/v1/namespaces/{namespace}",
	"/api/v1/namespaces/{namespace}/finalize",
	"/api/v1/namespaces/{namespace}/status",
	"/api/v1/nodes",
	"/api/v1/nodes/{name}",
	"/api/v1/nodes/{name}/proxy",
	"/api/v1/nodes/{name}/proxy/{path}",
	"/api/v1/nodes/{name}/status",
	"/api/v1/persistentvolumeclaims",
	"/api/v1/persistentvolumes",
	"/api/v1/persistentvolumes/{name}",
	"/api/v1/persistentvolumes/{name}/status",
	"/api/v1/pods",
	"/api/v1/podtemplates",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}/{path}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}/{path}",
	"/api/v1/proxy/nodes/{name}",
	"/api/v1/proxy/nodes/{name}/{path}",
	"/api/v1/replicationcontrollers",
	"/api/v1/resourcequotas",
	"/api/v1/secrets",
	"/api/v1/serviceaccounts",
	"/api/v1/services",
	"/api/v1/watch/configmaps",
	"/api/v1/watch/endpoints",
	"/api/v1/watch/events",
	"/api/v1/watch/limitranges",
	"/api/v1/watch/namespaces",
	"/api/v1/watch/namespaces/{namespace}/configmaps",
	"/api/v1/watch/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/watch/namespaces/{namespace}/endpoints",
	"/api/v1/watch/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/watch/namespaces/{namespace}/events",
	"/api/v1/watch/namespaces/{namespace}/events/{name}",
	"/api/v1/watch/namespaces/{namespace}/limitranges",
	"/api/v1/watch/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/watch/namespaces/{namespace}/pods",
	"/api/v1/watch/namespaces/{namespace}/pods/{name}",
	"/api/v1/watch/namespaces/{namespace}/podtemplates",
	"/api/v1/watch/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/watch/namespaces/{namespace}/secrets",
	"/api/v1/watch/namespaces/{namespace}/secrets/{name}",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/watch/namespaces/{namespace}/services",
	"/api/v1/watch/namespaces/{namespace}/services/{name}",
	"/api/v1/watch/namespaces/{namespace}",
	"/api/v1/watch/nodes",
	"/api/v1/watch/nodes/{name}",
	"/api/v1/watch/persistentvolumeclaims",
	"/api/v1/watch/persistentvolumes",
	"/api/v1/watch/persistentvolumes/{name}",
	"/api/v1/watch/pods",
	"/api/v1/watch/podtemplates",
	"/api/v1/watch/replicationcontrollers",
	"/api/v1/watch/resourcequotas",
	"/api/v1/watch/secrets",
	"/api/v1/watch/serviceaccounts",
	"/api/v1/watch/services",
	"/apis/apps/v1beta1/",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}/status",
	"/apis/apps/v1beta1/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/watch/statefulsets",
	"/apis/authentication.k8s.io/v1beta1/",
	"/apis/authentication.k8s.io/v1beta1/tokenreviews",
	"/apis/authorization.k8s.io/v1beta1/",
	"/apis/authorization.k8s.io/v1beta1/namespaces/{namespace}/localsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/selfsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/subjectaccessreviews",
	"/apis/autoscaling/v1/",
	"/apis/autoscaling/v1/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/autoscaling/v1/watch/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/batch/v1/",
	"/apis/batch/v1/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v1/watch/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/",
	"/apis/batch/v2alpha1/cronjobs",
	"/apis/batch/v2alpha1/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}/status",
	"/apis/batch/v2alpha1/scheduledjobs",
	"/apis/batch/v2alpha1/watch/cronjobs",
	"/apis/batch/v2alpha1/watch/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/watch/scheduledjobs",
	"/apis/certificates.k8s.io/v1alpha1/",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/approval",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/status",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests/{name}",
	"/apis/extensions/v1beta1/",
	"/apis/extensions/v1beta1/daemonsets",
	"/apis/extensions/v1beta1/deployments",
	"/apis/extensions/v1beta1/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/ingresses",
	"/apis/extensions/v1beta1/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/rollback",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/apis/extensions/v1beta1/networkpolicies",
	"/apis/extensions/v1beta1/replicasets",
	"/apis/extensions/v1beta1/thirdpartyresources",
	"/apis/extensions/v1beta1/thirdpartyresources/{name}",
	"/apis/extensions/v1beta1/watch/daemonsets",
	"/apis/extensions/v1beta1/watch/deployments",
	"/apis/extensions/v1beta1/watch/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/ingresses",
	"/apis/extensions/v1beta1/watch/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/watch/networkpolicies",
	"/apis/extensions/v1beta1/watch/replicasets",
	"/apis/extensions/v1beta1/watch/thirdpartyresources",
	"/apis/extensions/v1beta1/watch/thirdpartyresources/{name}",
	"/apis/policy/v1beta1/",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}/status",
	"/apis/policy/v1beta1/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/watch/poddisruptionbudgets",
	"/apis/rbac.authorization.k8s.io/v1alpha1/",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/roles",
	"/apis/storage.k8s.io/v1beta1/",
	"/apis/storage.k8s.io/v1beta1/storageclasses",
	"/apis/storage.k8s.io/v1beta1/storageclasses/{name}",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses/{name}",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/{name}",
	"/apis/metrics.k8s.io/v1beta1/pods",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/",
	"/apis/metrics.k8s.io/v1beta1/nodes",
	"/apis/metrics.k8s.io/v1beta1/nodes/{name}",
}

var apiSetV111 = []string{
	"/api/v1/",
	"/api/v1/componentstatuses",
	"/api/v1/componentstatuses/{name}",
	"/api/v1/configmaps",
	"/api/v1/endpoints",
	"/api/v1/events",
	"/api/v1/limitranges",
	"/api/v1/namespaces",
	"/api/v1/namespaces/{namespace}/bindings",
	"/api/v1/namespaces/{namespace}/configmaps",
	"/api/v1/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/namespaces/{namespace}/endpoints",
	"/api/v1/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/namespaces/{namespace}/events",
	"/api/v1/namespaces/{namespace}/events/{name}",
	"/api/v1/namespaces/{namespace}/limitranges",
	"/api/v1/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}/status",
	"/api/v1/namespaces/{namespace}/pods",
	"/api/v1/namespaces/{namespace}/pods/{name}",
	"/api/v1/namespaces/{namespace}/pods/{name}/attach",
	"/api/v1/namespaces/{namespace}/pods/{name}/binding",
	"/api/v1/namespaces/{namespace}/pods/{name}/eviction",
	"/api/v1/namespaces/{namespace}/pods/{name}/exec",
	"/api/v1/namespaces/{namespace}/pods/{name}/log",
	"/api/v1/namespaces/{namespace}/pods/{name}/portforward",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/pods/{name}/status",
	"/api/v1/namespaces/{namespace}/podtemplates",
	"/api/v1/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/status",
	"/api/v1/namespaces/{namespace}/resourcequotas",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}/status",
	"/api/v1/namespaces/{namespace}/secrets",
	"/api/v1/namespaces/{namespace}/secrets/{name}",
	"/api/v1/namespaces/{namespace}/serviceaccounts",
	"/api/v1/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/namespaces/{namespace}/services",
	"/api/v1/namespaces/{namespace}/services/{name}",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/services/{name}/status",
	"/api/v1/namespaces/{namespace}",
	"/api/v1/namespaces/{namespace}/finalize",
	"/api/v1/namespaces/{namespace}/status",
	"/api/v1/nodes",
	"/api/v1/nodes/{name}",
	"/api/v1/nodes/{name}/proxy",
	"/api/v1/nodes/{name}/proxy/{path}",
	"/api/v1/nodes/{name}/status",
	"/api/v1/persistentvolumeclaims",
	"/api/v1/persistentvolumes",
	"/api/v1/persistentvolumes/{name}",
	"/api/v1/persistentvolumes/{name}/status",
	"/api/v1/pods",
	"/api/v1/podtemplates",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}/{path}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}/{path}",
	"/api/v1/proxy/nodes/{name}",
	"/api/v1/proxy/nodes/{name}/{path}",
	"/api/v1/replicationcontrollers",
	"/api/v1/resourcequotas",
	"/api/v1/secrets",
	"/api/v1/serviceaccounts",
	"/api/v1/services",
	"/api/v1/watch/configmaps",
	"/api/v1/watch/endpoints",
	"/api/v1/watch/events",
	"/api/v1/watch/limitranges",
	"/api/v1/watch/namespaces",
	"/api/v1/watch/namespaces/{namespace}/configmaps",
	"/api/v1/watch/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/watch/namespaces/{namespace}/endpoints",
	"/api/v1/watch/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/watch/namespaces/{namespace}/events",
	"/api/v1/watch/namespaces/{namespace}/events/{name}",
	"/api/v1/watch/namespaces/{namespace}/limitranges",
	"/api/v1/watch/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/watch/namespaces/{namespace}/pods",
	"/api/v1/watch/namespaces/{namespace}/pods/{name}",
	"/api/v1/watch/namespaces/{namespace}/podtemplates",
	"/api/v1/watch/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/watch/namespaces/{namespace}/secrets",
	"/api/v1/watch/namespaces/{namespace}/secrets/{name}",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/watch/namespaces/{namespace}/services",
	"/api/v1/watch/namespaces/{namespace}/services/{name}",
	"/api/v1/watch/namespaces/{namespace}",
	"/api/v1/watch/nodes",
	"/api/v1/watch/nodes/{name}",
	"/api/v1/watch/persistentvolumeclaims",
	"/api/v1/watch/persistentvolumes",
	"/api/v1/watch/persistentvolumes/{name}",
	"/api/v1/watch/pods",
	"/api/v1/watch/podtemplates",
	"/api/v1/watch/replicationcontrollers",
	"/api/v1/watch/resourcequotas",
	"/api/v1/watch/secrets",
	"/api/v1/watch/serviceaccounts",
	"/api/v1/watch/services",
	"/apis/apps/v1beta1/",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/namespaces/{namespace}/statefulsets/{name}/status",
	"/apis/apps/v1beta1/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1beta1/watch/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1beta1/watch/statefulsets",
	"/apis/authentication.k8s.io/v1beta1/",
	"/apis/authentication.k8s.io/v1beta1/tokenreviews",
	"/apis/authorization.k8s.io/v1beta1/",
	"/apis/authorization.k8s.io/v1beta1/namespaces/{namespace}/localsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/selfsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1beta1/subjectaccessreviews",
	"/apis/autoscaling/v1/",
	"/apis/autoscaling/v1/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/autoscaling/v1/watch/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/batch/v1/",
	"/apis/batch/v1/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v1/watch/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/",
	"/apis/batch/v2alpha1/cronjobs",
	"/apis/batch/v2alpha1/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/cronjobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/namespaces/{namespace}/scheduledjobs/{name}/status",
	"/apis/batch/v2alpha1/scheduledjobs",
	"/apis/batch/v2alpha1/watch/cronjobs",
	"/apis/batch/v2alpha1/watch/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v2alpha1/watch/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v2alpha1/watch/scheduledjobs",
	"/apis/certificates.k8s.io/v1alpha1/",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/approval",
	"/apis/certificates.k8s.io/v1alpha1/certificatesigningrequests/{name}/status",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1alpha1/watch/certificatesigningrequests/{name}",
	"/apis/extensions/v1beta1/",
	"/apis/extensions/v1beta1/daemonsets",
	"/apis/extensions/v1beta1/deployments",
	"/apis/extensions/v1beta1/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/ingresses",
	"/apis/extensions/v1beta1/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/rollback",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/apis/extensions/v1beta1/networkpolicies",
	"/apis/extensions/v1beta1/replicasets",
	"/apis/extensions/v1beta1/thirdpartyresources",
	"/apis/extensions/v1beta1/thirdpartyresources/{name}",
	"/apis/extensions/v1beta1/watch/daemonsets",
	"/apis/extensions/v1beta1/watch/deployments",
	"/apis/extensions/v1beta1/watch/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/ingresses",
	"/apis/extensions/v1beta1/watch/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/watch/networkpolicies",
	"/apis/extensions/v1beta1/watch/replicasets",
	"/apis/extensions/v1beta1/watch/thirdpartyresources",
	"/apis/extensions/v1beta1/watch/thirdpartyresources/{name}",
	"/apis/policy/v1beta1/",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}/status",
	"/apis/policy/v1beta1/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/watch/poddisruptionbudgets",
	"/apis/rbac.authorization.k8s.io/v1alpha1/",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1alpha1/watch/roles",
	"/apis/storage.k8s.io/v1beta1/",
	"/apis/storage.k8s.io/v1beta1/storageclasses",
	"/apis/storage.k8s.io/v1beta1/storageclasses/{name}",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses",
	"/apis/storage.k8s.io/v1beta1/watch/storageclasses/{name}",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/{name}",
	"/apis/metrics.k8s.io/v1beta1/pods",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/",
	"/apis/metrics.k8s.io/v1beta1/nodes",
	"/apis/metrics.k8s.io/v1beta1/nodes/{name}",
}

var apiSetV112 = []string{
	"/api/v1/",
	"/api/v1/componentstatuses",
	"/api/v1/componentstatuses/{name}",
	"/api/v1/configmaps",
	"/api/v1/endpoints",
	"/api/v1/events",
	"/api/v1/limitranges",
	"/api/v1/namespaces",
	"/api/v1/namespaces/{namespace}/bindings",
	"/api/v1/namespaces/{namespace}/configmaps",
	"/api/v1/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/namespaces/{namespace}/endpoints",
	"/api/v1/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/namespaces/{namespace}/events",
	"/api/v1/namespaces/{namespace}/events/{name}",
	"/api/v1/namespaces/{namespace}/limitranges",
	"/api/v1/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/namespaces/{namespace}/persistentvolumeclaims/{name}/status",
	"/api/v1/namespaces/{namespace}/pods",
	"/api/v1/namespaces/{namespace}/pods/{name}",
	"/api/v1/namespaces/{namespace}/pods/{name}/attach",
	"/api/v1/namespaces/{namespace}/pods/{name}/binding",
	"/api/v1/namespaces/{namespace}/pods/{name}/eviction",
	"/api/v1/namespaces/{namespace}/pods/{name}/exec",
	"/api/v1/namespaces/{namespace}/pods/{name}/log",
	"/api/v1/namespaces/{namespace}/pods/{name}/portforward",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy",
	"/api/v1/namespaces/{namespace}/pods/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/pods/{name}/status",
	"/api/v1/namespaces/{namespace}/podtemplates",
	"/api/v1/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/api/v1/namespaces/{namespace}/replicationcontrollers/{name}/status",
	"/api/v1/namespaces/{namespace}/resourcequotas",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/namespaces/{namespace}/resourcequotas/{name}/status",
	"/api/v1/namespaces/{namespace}/secrets",
	"/api/v1/namespaces/{namespace}/secrets/{name}",
	"/api/v1/namespaces/{namespace}/serviceaccounts",
	"/api/v1/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/namespaces/{namespace}/services",
	"/api/v1/namespaces/{namespace}/services/{name}",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy",
	"/api/v1/namespaces/{namespace}/services/{name}/proxy/{path}",
	"/api/v1/namespaces/{namespace}/services/{name}/status",
	"/api/v1/namespaces/{namespace}",
	"/api/v1/namespaces/{namespace}/finalize",
	"/api/v1/namespaces/{namespace}/status",
	"/api/v1/nodes",
	"/api/v1/nodes/{name}",
	"/api/v1/nodes/{name}/proxy",
	"/api/v1/nodes/{name}/proxy/{path}",
	"/api/v1/nodes/{name}/status",
	"/api/v1/persistentvolumeclaims",
	"/api/v1/persistentvolumes",
	"/api/v1/persistentvolumes/{name}",
	"/api/v1/persistentvolumes/{name}/status",
	"/api/v1/pods",
	"/api/v1/podtemplates",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}",
	"/api/v1/proxy/namespaces/{namespace}/pods/{name}/{path}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}",
	"/api/v1/proxy/namespaces/{namespace}/services/{name}/{path}",
	"/api/v1/proxy/nodes/{name}",
	"/api/v1/proxy/nodes/{name}/{path}",
	"/api/v1/replicationcontrollers",
	"/api/v1/resourcequotas",
	"/api/v1/secrets",
	"/api/v1/serviceaccounts",
	"/api/v1/services",
	"/api/v1/watch/configmaps",
	"/api/v1/watch/endpoints",
	"/api/v1/watch/events",
	"/api/v1/watch/limitranges",
	"/api/v1/watch/namespaces",
	"/api/v1/watch/namespaces/{namespace}/configmaps",
	"/api/v1/watch/namespaces/{namespace}/configmaps/{name}",
	"/api/v1/watch/namespaces/{namespace}/endpoints",
	"/api/v1/watch/namespaces/{namespace}/endpoints/{name}",
	"/api/v1/watch/namespaces/{namespace}/events",
	"/api/v1/watch/namespaces/{namespace}/events/{name}",
	"/api/v1/watch/namespaces/{namespace}/limitranges",
	"/api/v1/watch/namespaces/{namespace}/limitranges/{name}",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims",
	"/api/v1/watch/namespaces/{namespace}/persistentvolumeclaims/{name}",
	"/api/v1/watch/namespaces/{namespace}/pods",
	"/api/v1/watch/namespaces/{namespace}/pods/{name}",
	"/api/v1/watch/namespaces/{namespace}/podtemplates",
	"/api/v1/watch/namespaces/{namespace}/podtemplates/{name}",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers",
	"/api/v1/watch/namespaces/{namespace}/replicationcontrollers/{name}",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas",
	"/api/v1/watch/namespaces/{namespace}/resourcequotas/{name}",
	"/api/v1/watch/namespaces/{namespace}/secrets",
	"/api/v1/watch/namespaces/{namespace}/secrets/{name}",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts",
	"/api/v1/watch/namespaces/{namespace}/serviceaccounts/{name}",
	"/api/v1/watch/namespaces/{namespace}/services",
	"/api/v1/watch/namespaces/{namespace}/services/{name}",
	"/api/v1/watch/namespaces/{namespace}",
	"/api/v1/watch/nodes",
	"/api/v1/watch/nodes/{name}",
	"/api/v1/watch/persistentvolumeclaims",
	"/api/v1/watch/persistentvolumes",
	"/api/v1/watch/persistentvolumes/{name}",
	"/api/v1/watch/pods",
	"/api/v1/watch/podtemplates",
	"/api/v1/watch/replicationcontrollers",
	"/api/v1/watch/resourcequotas",
	"/api/v1/watch/secrets",
	"/api/v1/watch/serviceaccounts",
	"/api/v1/watch/services",
	"/apis/apps/v1/",
	"/apis/apps/v1/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1/namespaces/{namespace}/statefulsets/{name}/status",
	"/apis/apps/v1/statefulsets",
	"/apis/apps/v1/watch/namespaces/{namespace}/statefulsets",
	"/apis/apps/v1/watch/namespaces/{namespace}/statefulsets/{name}",
	"/apis/apps/v1/watch/statefulsets",
	"/apis/authentication.k8s.io/v1/",
	"/apis/authentication.k8s.io/v1/tokenreviews",
	"/apis/authorization.k8s.io/v1/",
	"/apis/authorization.k8s.io/v1/namespaces/{namespace}/localsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1/selfsubjectaccessreviews",
	"/apis/authorization.k8s.io/v1/subjectaccessreviews",
	"/apis/autoscaling/v1/",
	"/apis/autoscaling/v1/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/autoscaling/v1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/autoscaling/v1/watch/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/autoscaling/v1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/batch/v1/",
	"/apis/batch/v1/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v1/watch/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/",
	"/apis/batch/v1/cronjobs",
	"/apis/batch/v1/jobs",
	"/apis/batch/v1/namespaces/{namespace}/cronjobs",
	"/apis/batch/v1/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/cronjobs/{name}/status",
	"/apis/batch/v1/namespaces/{namespace}/jobs",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/batch/v1/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v1/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v1/namespaces/{namespace}/scheduledjobs/{name}/status",
	"/apis/batch/v1/scheduledjobs",
	"/apis/batch/v1/watch/cronjobs",
	"/apis/batch/v1/watch/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/cronjobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/cronjobs/{name}",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/batch/v1/watch/namespaces/{namespace}/scheduledjobs",
	"/apis/batch/v1/watch/namespaces/{namespace}/scheduledjobs/{name}",
	"/apis/batch/v1/watch/scheduledjobs",
	"/apis/certificates.k8s.io/v1beta1/",
	"/apis/certificates.k8s.io/v1beta1/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1beta1/certificatesigningrequests/{name}",
	"/apis/certificates.k8s.io/v1beta1/certificatesigningrequests/{name}/approval",
	"/apis/certificates.k8s.io/v1beta1/certificatesigningrequests/{name}/status",
	"/apis/certificates.k8s.io/v1beta1/watch/certificatesigningrequests",
	"/apis/certificates.k8s.io/v1beta1/watch/certificatesigningrequests/{name}",
	"/apis/extensions/v1beta1/",
	"/apis/extensions/v1beta1/daemonsets",
	"/apis/extensions/v1beta1/deployments",
	"/apis/extensions/v1beta1/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/ingresses",
	"/apis/extensions/v1beta1/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/daemonsets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/rollback",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/deployments/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/horizontalpodautoscalers/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/ingresses/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/jobs/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/scale",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicasets/{name}/status",
	"/apis/extensions/v1beta1/namespaces/{namespace}/replicationcontrollers/{name}/scale",
	"/apis/extensions/v1beta1/networkpolicies",
	"/apis/extensions/v1beta1/replicasets",
	"/apis/extensions/v1beta1/thirdpartyresources",
	"/apis/extensions/v1beta1/thirdpartyresources/{name}",
	"/apis/extensions/v1beta1/watch/daemonsets",
	"/apis/extensions/v1beta1/watch/deployments",
	"/apis/extensions/v1beta1/watch/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/ingresses",
	"/apis/extensions/v1beta1/watch/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/daemonsets/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/deployments/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/horizontalpodautoscalers/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/ingresses/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/jobs/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/networkpolicies/{name}",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets",
	"/apis/extensions/v1beta1/watch/namespaces/{namespace}/replicasets/{name}",
	"/apis/extensions/v1beta1/watch/networkpolicies",
	"/apis/extensions/v1beta1/watch/replicasets",
	"/apis/extensions/v1beta1/watch/thirdpartyresources",
	"/apis/extensions/v1beta1/watch/thirdpartyresources/{name}",
	"/apis/policy/v1beta1/",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/namespaces/{namespace}/poddisruptionbudgets/{name}/status",
	"/apis/policy/v1beta1/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets",
	"/apis/policy/v1beta1/watch/namespaces/{namespace}/poddisruptionbudgets/{name}",
	"/apis/policy/v1beta1/watch/poddisruptionbudgets",
	"/apis/rbac.authorization.k8s.io/v1/",
	"/apis/rbac.authorization.k8s.io/v1/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1/roles",
	"/apis/rbac.authorization.k8s.io/v1/watch/clusterrolebindings",
	"/apis/rbac.authorization.k8s.io/v1/watch/clusterrolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1/watch/clusterroles",
	"/apis/rbac.authorization.k8s.io/v1/watch/clusterroles/{name}",
	"/apis/rbac.authorization.k8s.io/v1/watch/namespaces/{namespace}/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1/watch/namespaces/{namespace}/rolebindings/{name}",
	"/apis/rbac.authorization.k8s.io/v1/watch/namespaces/{namespace}/roles",
	"/apis/rbac.authorization.k8s.io/v1/watch/namespaces/{namespace}/roles/{name}",
	"/apis/rbac.authorization.k8s.io/v1/watch/rolebindings",
	"/apis/rbac.authorization.k8s.io/v1/watch/roles",
	"/apis/storage.k8s.io/v1/",
	"/apis/storage.k8s.io/v1/storageclasses",
	"/apis/storage.k8s.io/v1/storageclasses/{name}",
	"/apis/storage.k8s.io/v1/watch/storageclasses",
	"/apis/storage.k8s.io/v1/watch/storageclasses/{name}",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/{name}",
	"/apis/metrics.k8s.io/v1beta1/pods",
	"/apis/metrics.k8s.io/v1beta1/namespaces/{namespace}/pods/",
	"/apis/metrics.k8s.io/v1beta1/nodes",
	"/apis/metrics.k8s.io/v1beta1/nodes/{name}",
}
