/*
 * Tencent is pleased to support the open source community by making Blueking Container Service available.
 * Copyright (C) 2019 THL A29 Limited, a Tencent company. All rights reserved.
 * Licensed under the MIT License (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * http://opensource.org/licenses/MIT
 * Unless required by applicable law or agreed to in writing, software distributed under
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
// Code generated by client-gen. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	v1 "github.com/Tencent/bk-bcs/bcs-k8s/kubernetes/apis/cloud/v1"
	scheme "github.com/Tencent/bk-bcs/bcs-k8s/kubernetes/generated/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// NodeNetworksGetter has a method to return a NodeNetworkInterface.
// A group's client should implement this interface.
type NodeNetworksGetter interface {
	NodeNetworks(namespace string) NodeNetworkInterface
}

// NodeNetworkInterface has methods to work with NodeNetwork resources.
type NodeNetworkInterface interface {
	Create(ctx context.Context, nodeNetwork *v1.NodeNetwork, opts metav1.CreateOptions) (*v1.NodeNetwork, error)
	Update(ctx context.Context, nodeNetwork *v1.NodeNetwork, opts metav1.UpdateOptions) (*v1.NodeNetwork, error)
	UpdateStatus(ctx context.Context, nodeNetwork *v1.NodeNetwork, opts metav1.UpdateOptions) (*v1.NodeNetwork, error)
	Delete(ctx context.Context, name string, opts metav1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error
	Get(ctx context.Context, name string, opts metav1.GetOptions) (*v1.NodeNetwork, error)
	List(ctx context.Context, opts metav1.ListOptions) (*v1.NodeNetworkList, error)
	Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.NodeNetwork, err error)
	NodeNetworkExpansion
}

// nodeNetworks implements NodeNetworkInterface
type nodeNetworks struct {
	client rest.Interface
	ns     string
}

// newNodeNetworks returns a NodeNetworks
func newNodeNetworks(c *CloudV1Client, namespace string) *nodeNetworks {
	return &nodeNetworks{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the nodeNetwork, and returns the corresponding nodeNetwork object, and an error if there is any.
func (c *nodeNetworks) Get(ctx context.Context, name string, options metav1.GetOptions) (result *v1.NodeNetwork, err error) {
	result = &v1.NodeNetwork{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("nodenetworks").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of NodeNetworks that match those selectors.
func (c *nodeNetworks) List(ctx context.Context, opts metav1.ListOptions) (result *v1.NodeNetworkList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.NodeNetworkList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("nodenetworks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested nodeNetworks.
func (c *nodeNetworks) Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("nodenetworks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a nodeNetwork and creates it.  Returns the server's representation of the nodeNetwork, and an error, if there is any.
func (c *nodeNetworks) Create(ctx context.Context, nodeNetwork *v1.NodeNetwork, opts metav1.CreateOptions) (result *v1.NodeNetwork, err error) {
	result = &v1.NodeNetwork{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("nodenetworks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(nodeNetwork).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a nodeNetwork and updates it. Returns the server's representation of the nodeNetwork, and an error, if there is any.
func (c *nodeNetworks) Update(ctx context.Context, nodeNetwork *v1.NodeNetwork, opts metav1.UpdateOptions) (result *v1.NodeNetwork, err error) {
	result = &v1.NodeNetwork{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("nodenetworks").
		Name(nodeNetwork.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(nodeNetwork).
		Do(ctx).
		Into(result)
	return
}

// UpdateStatus was generated because the type contains a Status member.
// Add a +genclient:noStatus comment above the type to avoid generating UpdateStatus().
func (c *nodeNetworks) UpdateStatus(ctx context.Context, nodeNetwork *v1.NodeNetwork, opts metav1.UpdateOptions) (result *v1.NodeNetwork, err error) {
	result = &v1.NodeNetwork{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("nodenetworks").
		Name(nodeNetwork.Name).
		SubResource("status").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(nodeNetwork).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the nodeNetwork and deletes it. Returns an error if one occurs.
func (c *nodeNetworks) Delete(ctx context.Context, name string, opts metav1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("nodenetworks").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *nodeNetworks) DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("nodenetworks").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched nodeNetwork.
func (c *nodeNetworks) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.NodeNetwork, err error) {
	result = &v1.NodeNetwork{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("nodenetworks").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
