/*
 * Tencent is pleased to support the open source community by making Blueking Container Service available.
 * Copyright (C) 2019 THL A29 Limited, a Tencent company. All rights reserved.
 * Licensed under the MIT License (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * http://opensource.org/licenses/MIT
 * Unless required by applicable law or agreed to in writing, software distributed under,
 * the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language governing permissions and
 * limitations under the License.
 */

package clustercredential

import (
	"context"

	"github.com/Tencent/bk-bcs/bcs-common/common/blog"
	"github.com/Tencent/bk-bcs/bcs-common/common/modules"
	cmproto "github.com/Tencent/bk-bcs/bcs-services/bcs-cluster-manager/api/clustermanager"
	"github.com/Tencent/bk-bcs/bcs-services/bcs-cluster-manager/internal/store"
	"github.com/Tencent/bk-bcs/bcs-services/bcs-cluster-manager/internal/types"
)

// UpdateAction update action for online cluster credential
type UpdateAction struct {
	ctx   context.Context
	model store.ClusterManagerModel
	req   *cmproto.UpdateClusterCredentialReq
	resp  *cmproto.UpdateClusterCredentialResp
}

// NewUpdateAction create update action for online cluster credential
func NewUpdateAction(model store.ClusterManagerModel) *UpdateAction {
	return &UpdateAction{
		model: model,
	}
}

func (ua *UpdateAction) validate() error {
	if err := ua.req.Validate(); err != nil {
		return err
	}
	return nil
}

func (ua *UpdateAction) updateCredential() error {
	newCredential := &types.ClusterCredential{
		ServerKey:     ua.req.ServerKey,
		ClusterID:     ua.req.ClusterID,
		ClientModule:  ua.req.ClientModule,
		ServerAddress: ua.req.ServerAddress,
		CaCertData:    ua.req.CaCertData,
		UserToken:     ua.req.UserToken,
		ConnectMode:   modules.BCSConnectModeTunnel,
	}
	if err := ua.model.PutClusterCredential(ua.ctx, newCredential); err != nil {
		return err
	}
	return nil
}

func (ua *UpdateAction) setResp(code uint64, msg string) {
	ua.resp.Code = code
	ua.resp.Message = msg
	ua.resp.Result = (code == types.BcsErrClusterManagerSuccess)
}

// Handle handle update cluster credential
func (ua *UpdateAction) Handle(
	ctx context.Context, req *cmproto.UpdateClusterCredentialReq, resp *cmproto.UpdateClusterCredentialResp) {

	if req == nil || resp == nil {
		blog.Errorf("update cluster credential failed, req or resp is empty")
		return
	}
	ua.ctx = ctx
	ua.req = req
	ua.resp = resp

	if err := ua.validate(); err != nil {
		ua.setResp(types.BcsErrClusterManagerInvalidParameter, err.Error())
		return
	}
	if err := ua.updateCredential(); err != nil {
		ua.setResp(types.BcsErrClusterManagerDBOperation, err.Error())
		return
	}
	ua.setResp(types.BcsErrClusterManagerSuccess, types.BcsErrClusterManagerSuccessStr)
	return
}
